--[[=============================================================================
    Commands received from the receiver proxy (ReceivedFromProxy)

    Copyright 2015 Control4 Corporation. All Rights Reserved.
===============================================================================]]

-- This macro is utilized to identify the version string of the driver template version used.
if (TEMPLATE_VERSION ~= nil) then
	TEMPLATE_VERSION.device_messages = "2015.03.31"
end

--=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-- Power Functions
--=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
--[[
	Proxy Command: CONNECT_OUTPUT
	Parameters:
		output: mod 1000 value of Output Connection id
--]]    
function CONNECT_OUTPUT(output)
    LogTrace("CONNECT_OUTPUT")
    local command_delay = tonumber(Properties["Power On Delay Seconds"])
    local delay_units = "SECONDS"
    local command    
    if (gControlMethod == "IR") then
		-- TODO: create packet/command to send to the device
		command = CMDS_IR_ZONES[output]["CONNECT_OUTPUT"]
		LogTrace("command = " .. command)
		PackAndQueueCommand("CONNECT_OUTPUT", command, command_delay, delay_units)
    else
		-- TODO: create packet/command to send to the device
		command = "p.1"
		LogTrace("command = " .. command)
		PackAndQueueCommand("CONNECT_OUTPUT", command, command_delay, delay_units)
		
		GetDeviceVolumeStatus(output)
    end
end

--[[
	Proxy Command: DISCONNECT_OUTPUT
	Parameters:
		output: mod 1000 value of Output Connection id
--]]
function DISCONNECT_OUTPUT(output)
    LogTrace("DISCONNECT_OUTPUT")
    local command_delay = tonumber(Properties["Power Off Delay Seconds"])
    local delay_units = "SECONDS"
    local command 
    if (gControlMethod == "IR") then
		-- TODO: create packet/command to send to the device
		command = CMDS_IR_ZONES[output]["DISCONNECT_OUTPUT"]
    else
	   command = "p.0" 
    end  
    LogTrace("command = " .. command)
    PackAndQueueCommand("DISCONNECT_OUTPUT", command, command_delay, delay_units)
end

--=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-- Input Selection Functions
--=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
--[[
	Proxy Command: SET_INPUT
	Parameters:
		idBinding: proxybindingid of proxy bound to input connection
		output: mod 1000 value of Output Connection id	
		input: mod 1000 value of Input Connection id
--]]
function SET_INPUT(idBinding, output, input)
	local command
	if (gControlMethod == "IR") then		
		-- TODO: create packet/command to send to the device
		command = tInputCommandMap_IR[output][tInputConnMapByID[input].Name]	
	else	
		command = "i."..input
	end
	LogTrace("command = " .. command)
	PackAndQueueCommand("SET_INPUT", command)
end

--=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-- Volume Control Functions
--=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
--[[
	Proxy Command: MUTE_OFF
	Parameters:
		output: mod 1000 value of Output Connection id	
--]]
function MUTE_OFF(output)
	local command
	if (gControlMethod == "IR") then		
		-- TODO: create packet/command to send to the device
		command = CMDS_IR_ZONES[output]["MUTE_OFF"]	
	else
		command = "m.0"
	end 	
	LogTrace("command = " .. command)
	PackAndQueueCommand("MUTE_OFF", command)
end

--[[
	Proxy Command: MUTE_ON
	Parameters:
		output: mod 1000 value of Output Connection id	
--]]
function MUTE_ON(output)
	local command
	if (gControlMethod == "IR") then			
		command = CMDS_IR_ZONES[output]["MUTE_ON"]	
	else
		command = "m.1"
	end 	
	LogTrace("command = " .. command)
	PackAndQueueCommand("MUTE_ON", command)		
end

--[[
	Proxy Command: MUTE_TOGGLE
	Parameters:
		output: mod 1000 value of Output Connection id	
--]]
function MUTE_TOGGLE(output)
	local command
	if (gControlMethod == "IR") then			
		command = CMDS_IR_ZONES[output]["MUTE_TOGGLE"]	
	else
		command = "m.t"
	end 
	LogTrace("command = " .. command)
	PackAndQueueCommand("MUTE_TOGGLE", command)
end

--[[
	Proxy Command: SET_VOLUME_LEVEL
	Parameters:
		output: mod 1000 value of Output Connection id	
		c4VolumeLevel: volume level to be set represented in Control4 scale (0-100)
--]]
function SET_VOLUME_LEVEL(output, c4VolumeLevel)
	local command = "v." .. c4VolumeLevel --c4VolumeLevel = deviceVolumeLevel
	LogTrace("command = " .. command)
	PackAndQueueCommand("SET_VOLUME_LEVEL", command)
end

--[[
	Helper Function: SET_VOLUME_LEVEL_DEVICE
	Parameters:
		output: mod 1000 value of Output Connection id	
		deviceVolumeLevel: volume level to be set represented in device scale (as sepcified in the device's control protocol)
--]]
function SET_VOLUME_LEVEL_DEVICE(output, deviceVolumeLevel)
	--Called from ContinueVolumeRamping()
	
	local command = "v." .. deviceVolumeLevel
	
	LogTrace("command = " .. command)
	local command_delay = tonumber(Properties["Volume Ramp Delay Milliseconds"])
	PackAndQueueCommand("SET_VOLUME_LEVEL_DEVICE", command, command_delay)
end

--[[
	Proxy Command: PULSE_VOL_DOWN
	Parameters:
		output: mod 1000 value of Output Connection id	
--]]
function PULSE_VOL_DOWN(output)
	local command
	if (gControlMethod == "IR") then		
		-- TODO: create packet/command to send to the device
		command = CMDS_IR_ZONES[output]["VOLUME_DOWN"]	
	else
		command = "v.d"
	end 		
	LogTrace("command = " .. command)
	local command_delay = tonumber(Properties["Volume Ramp Delay Milliseconds"])
	PackAndQueueCommand("PULSE_VOL_DOWN", command, command_delay)
end

--[[
	Proxy Command: PULSE_VOL_UP
	Parameters:
		output: mod 1000 value of Output Connection id	
--]]
function PULSE_VOL_UP(output)
	local command
	if (gControlMethod == "IR") then	
		-- TODO: create packet/command to send to the device
		command = CMDS_IR_ZONES[output]["VOLUME_UP"]	
	else
		command = "v.u"
	end 		
	LogTrace("command = " .. command)
	local command_delay = tonumber(Properties["Volume Ramp Delay Milliseconds"])
	PackAndQueueCommand("PULSE_VOL_UP", command, command_delay)
end


--=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-- Helper Functions
--=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

--[[
	Helper Function: SEND_COMMAND_FROM_COMMAND_TABLE
	Parameters:
		idBinding: proxy id	
		output: mod 1000 value of Output Connection id
		command_name: name of command to be sent
--]]
function SEND_COMMAND_FROM_COMMAND_TABLE(idBinding, output, command_name)
    local output_for_log = output or "nil"
    LogTrace("SEND_COMMAND_FROM_COMMAND_TABLE(), idBinding=" .. idBinding .. ", output=" .. output_for_log .. ", command_name=" .. command_name)
	
	-- TODO: create packet/command to send to the device
	local command = GetCommandFromCommandTable(idBinding, output, command_name)
	
	if (command == nil) then
		LogTrace("command is nil")
	else
		LogTrace("command = " .. command)
		PackAndQueueCommand(command_name, command)
	end			
end

--[[
	Helper Function: GetCommandFromCommandTable
	Parameters:
		idBinding: proxy id	
		output: mod 1000 value of Output Connection id
		command_name: name of command to be returned
--]]
function GetCommandFromCommandTable(idBinding, output, command_name)
	LogTrace("GetCommand()")
	local t = {}
	
	-- TODO: select/create appropriate commands table structure
	
	if (gControlMethod == "IR") then
		t = CMDS_IR
	else
		t = CMDS
	end	

	if (t[idBinding][command_name] ~= nil) then
		return t[idBinding][command_name]
	elseif (t[command_name] ~= nil) then
		return t[command_name]
	else
		LogWarn('GetCommandFromCommandTable: command not defined - '.. command_name)
		return nil
	end	
	

	--this code sample uses the "output" parameter for indexing the IR command table
	--not all proxies return an output parameter so this must be derived using the findOutputFromBindingID() and related functions
	if (gControlMethod == "IR") then
		t = CMDS_IR_ZONES
		if (output == nil) then
			output = findOutputFromBindingID(idBinding)
		end	
		if (t[output][strCommandName] ~= nil) then
			return t[output][strCommandName]
		elseif (t[strCommandName] ~= nil) then
			return t[strCommandName]
		else
			LogWarn('GetCommand(): Command not defined - '.. strCommandName)
			return nil
		end			
	else
		t = CMDS

		if (t[idBinding][strCommandName] ~= nil) then
			return t[idBinding][strCommandName]
		elseif (t[strCommandName] ~= nil) then
			return t[strCommandName]
		else
			LogWarn('GetCommand(): Command not defined - '.. strCommandName)
			return nil
		end		
		
	end		

	
end

--[[
	Helper Function: findOutputFromBindingID
	Parameters:
		idBinding: proxy id	
	Called from GetCommandFromCommandTable() when the deviceprotocol has multi-zone 
	command structures for commands which Control4 does not include an OUTPUT parameter
--]]
function findOutputFromBindingID(idBinding)
	-- TODO: edit tProxyInputNameMapByProxyID based upon the AV proxy connection modelling
	tProxyInputNameMapByProxyID = {
		[TUNER_PROXY_BINDINGID] = "AM/FM TUNER", 
		
	}
	
	LogTrace('findOutputFromBindingID: ' .. idBinding)
	local name = tProxyInputNameMapByProxyID[idBinding] 
	local output = findOutputIDFromInputName(name)
	return output
end

--[[
	Helper Function: findOutputIDFromInputName
	Parameters:
		inputName: input connection  name (this will be the input for the AV proxy connection)	
	Called from findOutputFromBindingID() when the device protocol has multi-zone 
	command structures for commands which Control4 does not include an OUTPUT parameter
--]]
function findOutputIDFromInputName(inputName)
    LogTrace('findOutputIDFromInputName: ' .. inputName)
    local output
    for j,k in pairs(gReceiverProxy._CurrentlySelectedInput) do
		if (tInputConnMapByID[k] ~= nil) then
			if (inputName==tInputConnMapByID[k].Name) then
				if (j==0) then 
					return j 
				else
					output = j
				end
			end	
		end
	end
    return output 
end

--[[
	Helper Function: GetDeviceVolumeStatus
--]]
function GetDeviceVolumeStatus(output)
    LOG:Trace("GetDeviceVolumeStatus(), output = " .. output)
	
	-- TODO: verify table entries in tVolumeQueryMap for all zones
	local command = "v.?"
	LOG:Trace("command = " .. command)
	PackAndQueueCommand("GetDeviceVolumeStatus: Volume", command)
	
	-- TODO: verify table entries in tMuteCommandMap for all zones, modify line below if needed
	command = "m.?"
	LOG:Trace("command = " .. command)
	PackAndQueueCommand("GetDeviceVolumeStatus: Mute", command)	
end

--[[
	Helper Function: GetDevicePowerStatus
--]]
function GetDevicePowerStatus(output)
    LOG:Trace("GetDevicePowerStatus()")
	
	-- TODO: verify table entry in Volume in QUERY table
	local command = "p.?"
	LOG:Trace("command = " .. command)
	PackAndQueueCommand("GetDevicePowerStatus: Volume", command)	
end
